/*---------------------------------------------------------------------------+
 |  Floating-point environment <fenv.h>                                      |
 |                                                                           |
 |                                                                           |
 | Copyright (C) 1996, 1997                                                  |
 |                     W. Metzenthen, 22 Parker St, Ormond, Vic 3163,        |
 |                     Australia.  E-mail   billm@suburbia.net               |
 |                                                                           |
 | See the files "README" and "COPYING" for further copyright and warranty   |
 | information.                                                              |
 |                                                                           |
 +---------------------------------------------------------------------------*/
  
/*---------------------------------------------------------------------------+
 |  History:                                                                 |
 |  10 Sep 1993: First version, based upon early NCEG docs.                  |
 |  29 Feb 1996: Second version, minor modifications to satisfy              |
 |               WG14/N513 X3J11/95-114 (Draft 12/21/95)                     |
 +---------------------------------------------------------------------------*/
 
 
#ifndef _FENV_H
#define _FENV_H
 
#define FE_INEXACT       0x20
#define FE_DIVBYZERO     0x04
#define FE_UNDERFLOW     0x10
#define FE_OVERFLOW      0x08
#define FE_INVALID       0x01
#define FE_ALL_EXCEPT    ( FE_INEXACT  | FE_DIVBYZERO | FE_UNDERFLOW                          | FE_OVERFLOW | FE_INVALID )
 
#define FE_TONEAREST     0x000
#define FE_UPWARD        0x800
#define FE_DOWNWARD      0x400
#define FE_TOWARDZERO    0xc00
 
#define FE_DFL_ENV       (&__default_fp_env)
 
/*  fenv_t: a type for representing the entire floating-point environment. */
/*  fenv_t is not defined in detail by NCEG.
    For Linux, we use the FPU environment:  */
struct __fenv {
  unsigned long control;
  unsigned long status;
  unsigned long tags;
  unsigned long ip_offset;
  unsigned long opcode;
  unsigned long data_offset;
  unsigned long op_sel;
};
typedef struct __fenv fenv_t;
 
extern const fenv_t __default_fp_env;
 
/*  fexcept_t: a type for representing the floating-point exception flags
    collectively, including any status associated with the flags. */
/*  fexcept_t is not defined in detail by NCEG.
    For Linux, it might be something like this:  */
struct __fexcept {
         int excepts;
         void *code_address;
         void *data_address;
};
typedef struct __fexcept fexcept_t;
 
__BEGIN_DECLS
void feclearexcept(int excepts);
void feraiseexcept(int excepts);
int fetestexcept(int excepts);
 
void fegetexceptflag(fexcept_t *flagp, int excepts);
void fesetexceptflag(const fexcept_t *flagp, int excepts);
 
int fegetround(void);
int fesetround(int round);
 
void fegetenv(fenv_t *envp);
int feholdexcept(fenv_t *envp);
void fesetenv(const fenv_t *envp);
void feupdateenv(const fenv_t *envp);
__END_DECLS
 
#ifdef NON_PORTABLE_FENV
 
#define FE_DENORM_OP     0x02    /* 80x87, Not mentioned by NCEG */
#define FE_STACK_FLT     0x40    /* 80x87, Not mentioned by NCEG */
#undef FE_ALL_EXCEPT
#define FE_ALL_EXCEPT    ( FE_INEXACT  | FE_DIVBYZERO | FE_UNDERFLOW                          | FE_OVERFLOW | FE_INVALID   | FE_DENORM_OP )
 
 
#define FE_FLTPREC       0x000
#define FE_INVALIDPREC   0x100
#define FE_DBLPREC       0x200
#define FE_LDBLPREC      0x300
 
/*
  These are non-standard functions.
  They will probably be removed in a future release.
  DO NOT USE if you require portability.
  */
#include <asm/sigcontext.h>
#if defined(__cplusplus)
/* My version of g++ doesn't like Linus' definition of asmlinkage */
#define _PERSONALITY_H
#endif
/* #include <linux/sched.h> rm by crc */ 
 
/*
  Change this to "#if 1" if you get warning and/or error messages
  from gcc about sigcontext_struct.
  The name of the structure has changed in <asm/sigcontext.h>, grrrh!.
  */
#if 1 /* %% Change this line as needed. %% */
#define sigcontext_struct sigcontext
#endif
 
__BEGIN_DECLS
 
typedef int (*ftrap_t)(struct sigcontext_struct *sig,
		       struct i387_hard_struct *fpu );
 
int fegetprecision(void);
int fesetprecision(int prec);
 
int fegetvector(ftrap_t *trap, int except);
int fesetvector(ftrap_t *trap, int except);
int fegettraps(void);
int fedisabletraps(int excepts);
int feenabletraps(int excepts);
 
void report_fpu_exception_stats(FILE *f);
 
 
#ifdef __SIGFPE_INTERNAL
struct fptraps {
  ftrap_t invalid, stack_flt, denorm_op,
    divbyzero, overflow, underflow, inexact;
  int trap_invalid, trap_stack_flt;
};
 
extern struct fptraps __trap_vectors;
extern void __set_sticky(int bits);
#endif /* __SIGFPE_INTERNAL */
 
__END_DECLS
 
#endif /* NON_PORTABLE_FENV */
 
#endif /* _FENV_H */
 
